@@ -15,7 +15,7 @@ require Exporter;
 @EXPORT    = qw( whois whois_config ); ### It's bad manners to export lots.
 @EXPORT_OK = qw( $OMIT_MSG $CHECK_FAIL $CACHE_DIR $CACHE_TIME $USE_CNAMES $TIMEOUT);
 
-$VERSION = '0.26';
+$VERSION = '0.34';
 
 ($OMIT_MSG, $CHECK_FAIL, $CACHE_DIR, $CACHE_TIME, $USE_CNAMES, $TIMEOUT) = (0) x 6;
 
@@ -126,20 +126,7 @@ sub finish {
     local ($_) = join("\n", @lines, "");
 
     if ($CHECK_FAIL > 1) {
-        return undef if
-	    /is unavailable/is ||
-	    /No entries found for the selected source/is ||
-	    /Not found:/s ||
-	    /No match\./s ||
-	    /is available/is ||
-	    /Not found/is && !/ your query returns "NOT FOUND"/ ||
-	    /No match for/is ||
-	    /No Objects Found/s ||
-	    /No domain records were found/s ||
-	    /No such domain/s ||
-	    /No entries found in the /s ||
-	    /Unable to find any information for your query/s ||
-	    /is not registered and may be available for registration/s;
+	return undef unless check_existance($_);
     }
 
     if ($OMIT_MSG > 1) {	
@@ -151,7 +138,7 @@ sub finish {
         s/NOTICE: Access to.+this policy.//is;
         s/The previous information.+completeness\.//s;
         s/NOTICE AND TERMS OF USE:.*modify these terms at any time\.//s;
-        s/TERMS OF USE:.*modify these terms at any time\.//s;
+        s/TERMS OF USE:.*?modify these terms at any time\.//s;
         s/NOTICE:.*expiration for this registration\.//s;
 
 	s/By submitting a WHOIS query.+?DOMAIN AVAILABILITY.\n?//s;
@@ -161,7 +148,7 @@ sub finish {
 	s/Whois Server Version \d+\.\d+.//is;
 	s/NeuStar,.+www.whois.us\.//is;
         s/\n?Domain names in the \.com, .+ detailed information.\n?//s;
-        s/\n?The Registry database .+?Registrars\.\n?//s;
+        s/\n?The Registry database .+?Registrars\.\n//s;
         s/\n?>>> Last update of .+? <<<\n?//;
         s/% .+?\n//gs;
     }
@@ -189,6 +176,7 @@ sub _whois {
     print $sock "$whoisquery\r\n";
     my @lines = <$sock>;
     close($sock);
+    my $answer = join '', @lines;
     if ($flag) {
         foreach (@lines) {
             $state ||= (/Registrar:/);
@@ -197,8 +185,8 @@ sub _whois {
                 next if (($newsrv) eq uc($srv));
                 return undef if (grep {$_ eq $newsrv} @$ary);
 		my $whois = eval { _whois($dom, $newsrv, $flag, [@$ary, $srv]) };
-		if ($@ && !$whois) {
-		    return join '', @lines;
+		if ($@ && !$whois || $whois && !check_existance($whois)) {
+		    return $answer;
 		}
                 return $whois;
             }
@@ -208,7 +196,7 @@ sub _whois {
                 return undef if (grep {$_ eq $newsrv} @$ary);
                 my $whois = eval { _whois($dom, $newsrv, $flag, [@$ary, $srv]) };
 		if ($@ && !$whois) {
-		    return join '', @lines;
+		    return $answer;
 		}
                 return $whois;
             }
@@ -230,6 +218,30 @@ sub dlen {
     return length($str) * (1 + $dotcount);
 }
 
+
+sub check_existance {
+    $_ = $_[0];
+
+    return undef if
+	/is unavailable/is ||
+	/No entries found for the selected source/is ||
+	/Not found:/s ||
+	/No match\./s ||
+	/is available for/is ||
+	/Not found/is &&
+	    !/ your query returns "NOT FOUND"/ &&
+	    !/Domain not found locally/ ||
+	/No match for/is ||
+	/No Objects Found/s ||
+	/No domain records were found/s ||
+	/No such domain/s ||
+	/No entries found in the /s ||
+	/Unable to find any information for your query/s ||
+	/is not registered and may be available for registration/s;
+    return 1;
+}
+
+
 # Preloaded methods go here.
 
 # Autoload methods go after =cut, and are processed by the autosplit program.
@@ -303,7 +315,7 @@ the search failed and return undef in such a case.
 B<IMPORTANT>: these checks merely use pattern matching; they will work
 on several servers but certainly not on all of them.
 
-(This features were contributed by Walery Studennikov B<despair@sama.ru>)
+(This features were contributed by Walery Studennikov B<despair@cpan.org>)
 
 =head1 AUTHOR
 
@@ -328,7 +340,7 @@ Added support for reverse lookup of IP addresses via the ARIN registry. (Alex Wi
 
 This will work now for RIPE addresses as well, according to a redirection from ARIN. (Philip Hands B<phil@uk.alcove.com>, Trevor Peirce B<trev@digitalcon.ca>)
 
-Added the pattern matching switches, (Walery Studennikov B<despair@sama.ru>)
+Added the pattern matching switches, (Walery Studennikov B<despair@cpan.org>)
 
 Modified pattern matching, added cache. (Tony L. Svanstrom B<tony@svanstrom.org>)
 
@@ -355,7 +367,7 @@ B<die> on L<perlfunc> about exception handling in Perl.
 Copyright 2000-2002 Ariel Brosh.
 Copyright 2003-2003 Gabor Szabo.
 Copyright 2003-2003 Corris Randall.
-Copyright 2003-2003 Walery Studennikov.
+Copyright 2003-2004 Walery Studennikov.
 
 This package is free software. You may redistribute it or modify it under
 the same terms as Perl itself.
